@@ -1,14 +1,44 @@
-bprobgHsSS <- function(params, BivD, nC, nu, H.n, y1, y2, y1.y2, y1.cy2, cy1.y2, cy1.cy2, cy1, X1, X2, weights=weights, X1.d2, X2.d2, pPen1=NULL, pPen2=NULL, sp=NULL, qu.mag=NULL, gp1, gp2, fp, l.sp1, l.sp2, K=NULL, n=NULL, N=NULL, cuid=NULL, uidf=NULL, masses=NULL, NGQ=NULL, dat1all=NULL, dat2all=NULL, W=NULL){
+bprobgHsSS <- function(params, BivD, nC, nu, gev.eq1, gev.eq2, shape1, shape2, H.n, y1.y2, y1.cy2, cy1.y2, cy1.cy2, cy1, X1, X2, weights=weights, X1.d2, X2.d2, pPen1=NULL, pPen2=NULL, sp=NULL, qu.mag=NULL, gp1, gp2, fp, l.sp1, l.sp2, K=NULL, n=NULL, N=NULL, cuid=NULL, uidf=NULL, masses=NULL, NGQ=NULL, dat1all=NULL, dat2all=NULL, W=NULL){
 
   eta1 <- X1%*%params[1:X1.d2]
   eta2 <- X2%*%params[(X1.d2+1):(X1.d2+X2.d2)]
-  p1 <- pnorm(eta1)
-  p2 <- pnorm(eta2)
-  d.n1   <- dnorm(eta1) 
-  d.n2   <- dnorm(eta2) 
+
+
+  if(gev.eq1==FALSE && gev.eq2==TRUE) {p1 <- pnorm(eta1); d.n1 <- dnorm(eta1)
+                                       p2 <- 1-pgev(-eta2,shape=shape2); d.n2 <- dgev(-eta2,shape=shape2)}   
+
+  if(gev.eq1==TRUE && gev.eq2==FALSE) {p1 <- 1-pgev(-eta1,shape=shape1); d.n1 <- dgev(-eta1,shape=shape1)
+                                       p2 <- pnorm(eta2); d.n2 <- dnorm(eta2)}
+
+
+  if(gev.eq1==TRUE && gev.eq2==TRUE) {p1 <- 1-pgev(-eta1,shape=shape1); d.n1 <- dgev(-eta1,shape=shape1)
+                                      p2 <- 1-pgev(-eta2,shape=shape2); d.n2 <- dgev(-eta2,shape=shape2)}
+
+
+  if(gev.eq1==FALSE && gev.eq2==FALSE) { p1 <- pnorm(eta1); d.n1 <- dnorm(eta1)
+                                         p2 <- pnorm(eta2); d.n2 <- dnorm(eta2) } 
+
   teta.st <- params[(X1.d2+X2.d2+1)]
   epsilon <- .Machine$double.eps*10^6
 
+  criteria <- c(0,1)
+  no.good <- apply(apply(cbind(p1,p2), c(1,2), `%in%`, criteria), 1, any)
+  good <- no.good==FALSE
+
+  p1 <- p1[good]
+  p2 <- p2[good]
+  d.n1 <- d.n1[good]
+  d.n2 <- d.n2[good]
+  eta1 <- eta1[good] 
+  eta2 <- eta2[good] 
+  X1 <- X1[good,]
+  X2 <- X2[good,]
+  y1.y2 <- y1.y2[good]
+  y1.cy2 <- y1.cy2[good]
+  cy1 <- cy1[good]
+  weights <- weights[good]
+
+
 ########################################################################################################
 
     if(BivD %in% c("N","T")      ){teta <- tanh(teta.st); if(teta %in% c(-1,1)) teta <- sign(teta)*0.9999999}
@@ -20,22 +50,22 @@ bprobgHsSS <- function(params, BivD, nC, nu, H.n, y1, y2, y1.y2, y1.cy2, cy1.y2,
     if(BivD %in% c("G0", "G180") ) teta <- exp(teta.st) + 1 
     if(BivD %in% c("G90","G270") ) teta <- -( exp(teta.st) + 1 ) 
 
-if(BivD=="N") C.copula <- pmax( abs(pnorm2( eta1, eta2, cov12=teta)), 1000*.Machine$double.eps ) else C.copula <- BiCopCDF(p1,p2, nC, par=teta, par2=nu)
+if(BivD=="N") C.copula <- pmax( abs(pbinorm( qnorm(p1), qnorm(p2), cov12=teta)), 1000*.Machine$double.eps ) else C.copula <- BiCopCDF(p1,p2, nC, par=teta, par2=nu)
 ########################################################################################################
 
   p11 <- pmax( C.copula, 1000*.Machine$double.eps )
   p10 <- pmax( p1 - p11, 1000*.Machine$double.eps )
-  p0  <- pmax( pnorm(-eta1), 1000*.Machine$double.eps )
+  p0  <- pmax( 1-p1, 1000*.Machine$double.eps )
 
   l.par <- weights*( y1.y2*log(p11) + y1.cy2*log(p10) + cy1*log(p0) ) 
 
-if(BivD=="N" && H.n==FALSE){
+if(BivD=="N" && H.n==FALSE && gev.eq1==FALSE && gev.eq2==FALSE){
 
   d.r  <- 1/sqrt( pmax(10000*.Machine$double.eps, 1-teta^2) )
   A   <- pnorm( (eta2-teta*eta1)*d.r ); A.c <- 1 - A
   B   <- pnorm( (eta1-teta*eta2)*d.r )
 
-  d.n1n2 <- dnorm2(eta1,eta2,rho=teta) 
+  d.n1n2 <- dbinorm(eta1,eta2,cov12=teta) 
   drh.drh.st   <- 4*exp(2*teta.st)/(exp(2*teta.st)+1)^2
   
   dl.dbe1 <- weights*d.n1*( y1.y2/p11*A  + y1.cy2/p10*A.c - cy1/p0 )  
@@ -53,21 +83,89 @@ if(BivD=="N" && H.n==FALSE){
 }else{
 
 
-  dH <- copgHs(eta1,eta2,p1,p2,teta,teta.st,BivD,nC,nu)
+dH <- copgHs(p1,p2,teta,teta.st,BivD,nC,nu)
 
 c.copula.be1   <- dH$c.copula.be1
 c.copula.be2   <- dH$c.copula.be2
 c.copula.theta <- dH$c.copula.theta 
                                          
-c.copula2.be1 <- dH$c.copula2.be1   
+c.copula2.be1 <- dH$c.copula2.be1  
+c.copula2.be2 <- dH$c.copula2.be2 
+
+
+if(gev.eq1==FALSE && gev.eq2==FALSE){
+
 bit1.b1b1 <- c.copula2.be1*(d.n1)^2-c.copula.be1*d.n1*eta1
 bit2.b1b1 <- -d.n1*eta1-bit1.b1b1
 bit3.b1b1 <- d.n1*(eta1*p0-d.n1)/p0^2
 
-c.copula2.be2 <- dH$c.copula2.be2 
 bit1.b2b2 <- c.copula2.be2*(d.n2)^2-c.copula.be2*d.n2*eta2
 bit2.b2b2 <- -bit1.b2b2
 
+}
+
+if(gev.eq1==FALSE && gev.eq2==TRUE){
+
+bit1.b1b1 <- c.copula2.be1*(d.n1)^2-c.copula.be1*d.n1*eta1
+bit2.b1b1 <- -d.n1*eta1-bit1.b1b1
+bit3.b1b1 <- d.n1*(eta1*p0-d.n1)/p0^2
+
+d1_d.n2 <- (1 + shape2 * (-eta2))^(-1/shape2 - 1) * (exp(-(1 + shape2 * (-eta2))^(-1/shape2)) * 
+    ((1 + shape2 * (-eta2))^((-1/shape2) - 1) * ((-1/shape2) * shape2))) - (1 + 
+    shape2 * (-eta2))^((-1/shape2 - 1) - 1) * ((-1/shape2 - 1) * shape2) * exp(-(1 + 
+    shape2 * (-eta2))^(-1/shape2))
+
+
+bit1.b2b2 <- c.copula2.be2*(d.n2)^2+c.copula.be2*d1_d.n2
+bit2.b2b2 <- -bit1.b2b2
+
+
+}
+
+
+if(gev.eq1==TRUE && gev.eq2==FALSE){
+
+
+d1_d.n1 <- (1 + shape1 * (-eta1))^(-1/shape1 - 1) * (exp(-(1 + shape1 * (-eta1))^(-1/shape1)) * 
+    ((1 + shape1 * (-eta1))^((-1/shape1) - 1) * ((-1/shape1) * shape1))) - (1 + 
+    shape1 * (-eta1))^((-1/shape1 - 1) - 1) * ((-1/shape1 - 1) * shape1) * exp(-(1 + 
+    shape1 * (-eta1))^(-1/shape1))
+
+bit1.b1b1 <- c.copula2.be1*(d.n1)^2+c.copula.be1*d1_d.n1 
+bit2.b1b1 <- d1_d.n1-bit1.b1b1 
+bit3.b1b1 <- (-p0*d1_d.n1-d.n1^2)/p0^2 
+
+
+bit1.b2b2 <- c.copula2.be2*(d.n2)^2-c.copula.be2*d.n2*eta2
+bit2.b2b2 <- -bit1.b2b2
+
+
+}
+
+if(gev.eq1==TRUE && gev.eq2==TRUE){
+
+d1_d.n1 <- (1 + shape1 * (-eta1))^(-1/shape1 - 1) * (exp(-(1 + shape1 * (-eta1))^(-1/shape1)) * 
+    ((1 + shape1 * (-eta1))^((-1/shape1) - 1) * ((-1/shape1) * shape1))) - (1 + 
+    shape1 * (-eta1))^((-1/shape1 - 1) - 1) * ((-1/shape1 - 1) * shape1) * exp(-(1 + 
+    shape1 * (-eta1))^(-1/shape1))
+
+bit1.b1b1 <- c.copula2.be1*(d.n1)^2+c.copula.be1*d1_d.n1 
+bit2.b1b1 <- d1_d.n1-bit1.b1b1 
+bit3.b1b1 <- (-p0*d1_d.n1-d.n1^2)/p0^2 
+
+
+d1_d.n2 <- (1 + shape2 * (-eta2))^(-1/shape2 - 1) * (exp(-(1 + shape2 * (-eta2))^(-1/shape2)) * 
+    ((1 + shape2 * (-eta2))^((-1/shape2) - 1) * ((-1/shape2) * shape2))) - (1 + 
+    shape2 * (-eta2))^((-1/shape2 - 1) - 1) * ((-1/shape2 - 1) * shape2) * exp(-(1 + 
+    shape2 * (-eta2))^(-1/shape2))
+
+
+bit1.b2b2 <- c.copula2.be2*(d.n2)^2+c.copula.be2*d1_d.n2
+bit2.b2b2 <- -bit1.b2b2
+
+}
+
+
 c.copula2.be1be2 <- dH$c.copula2.be1be2
 bit1.b1b2 <- c.copula2.be1be2 * d.n1 *d.n2
 bit2.b1b2 <- -bit1.b1b2
@@ -185,7 +283,7 @@ if( ( l.sp1==0 && l.sp2==0 ) || fp==TRUE) S.h <- S.h1 <- S.h2 <- 0
               dl.dbe1=dl.dbe1, dl.dbe2=dl.dbe2, dl.drho=dl.drho,
               d2l.be1.be1=d2l.be1.be1, d2l.be2.be2=d2l.be2.be2, 
               d2l.be1.be2=d2l.be1.be2, d2l.be1.rho=d2l.be1.rho,
-              d2l.be2.rho=d2l.be2.rho, d2l.rho.rho=d2l.rho.rho)     
+              d2l.be2.rho=d2l.be2.rho, d2l.rho.rho=d2l.rho.rho,good=good)     
 
 }
 
