@@ -5,7 +5,7 @@ if(x$RE==TRUE && x$RE.type=="NP"){ Kk1 <- x$K - 1; k.n <- 0}
 if(x$RE==TRUE && x$RE.type=="N"){  Kk1 <- 0; k.n <- 1}
 
  if(eq==1){ ss.plot <- x$gam1
-            ind <- (1+Kk1):(length(ss.plot$coefficients)+Kk1) 
+            ind <- (1:length(ss.plot$coefficients))+Kk1 
             } 
  if(eq==2){ ss.plot <- x$gam2
             fir <- length(x$gam1$coefficients) + 1 + 2*Kk1 + k.n
@@ -16,7 +16,7 @@ if(x$RE==TRUE && x$RE.type=="N"){  Kk1 <- 0; k.n <- 1}
            ss.plot$Vp <- x$Vb[ind,ind]
            ss.plot$edf <- diag(x$F)[ind]
 
-  plot.gam(ss.plot, ...)
+  plot.gam(ss.plot, ...)  # recall that with GEV this function is not good when plotting on the response scale
        
 }
 
