@@ -1,10 +1,14 @@
-working.comp <- function(x,X1=X1,X2=X2,X1.d2=X1.d2,X2.d2=X2.d2,n=n){
+working.comp <- function(x,X1=X1,X2=X2,X1.d2=X1.d2,X2.d2=X2.d2){
 
   e.par <- x$argument
   e.par <- e.par[-length(e.par)]
+  good <- x$good
+  n <- length(good)
   X <- rW.X <- matrix(0,2*n,(X1.d2+X2.d2))
   D <- rW.Z <- matrix(0,2*n,1)
   j <- 1
+  X1 <- X1[good,] 
+  X2 <- X2[good,] 
 
     for(i in seq(1,(2*n-1),by=2)) {
       X[i,1:X1.d2]                   <- X1[j,]
